// This software is made by  (c)Elektor Labs

#include <U8glib.h>
#include "HX711.h"

#define DOUT  A3
#define CLK   A2

HX711 scale(DOUT, CLK);
U8GLIB_SSD1306_128X64 u8g(7, 6, 14, 4, 5);  //scl,sda,cs(unused),dc,res

//variable declaration
float calibration_factor = -7050.0; //This value is obtained using the SparkFun_HX711_zeroibration sketch
float gram;
float ounce;
float factor;
int button1 = 2;
int button2 = 3;
int bar = 0;
bool button_zero = 0;
bool button_hold = 0;
uint8_t c[] = //copyright symbol
{
  0b00000011,0b11100000,
  0b00001100,0b00011000,
  0b00010000,0b10000100,
  0b00100011,0b01100010,
  0b00100100,0b00010010,
  0b01001000,0b00000001,
  0b01001000,0b00000001,
  0b01010000,0b00000001,
  0b01001000,0b00000001,
  0b01001000,0b00000001,
  0b00100100,0b00010010,
  0b00100011,0b01100010,
  0b00010000,0b10000100,
  0b00001100,0b00011000,
  0b00000011,0b11100000,
};

void setup() 
{
  Serial.begin(9600);

  //calibrate the the scale of the hx711 and reset the zero point
  scale.set_scale(calibration_factor);
  scale.tare();

  //set the interupts for the buttons
  pinMode(button1, INPUT);
  attachInterrupt(digitalPinToInterrupt(button1), buttonPressed1, FALLING);
  pinMode(button2, INPUT);
  attachInterrupt(digitalPinToInterrupt(button2), buttonPressed2, FALLING);

  //calibrate the OLED screen and draw the initial screen
  u8g.setColorIndex(1); 
  u8g.firstPage();  
  do {
    draw(0);
  } while(u8g.nextPage());
  u8g.firstPage();  
  delay(1000);
}

//interrupt functions for the buttons
void buttonPressed1()
{
  button_zero = 1;
}
void buttonPressed2()
{
  button_hold = 1;
}

//main function to draw the pages on the OLED screen
void draw(int page) 
{
  //the outer border of the screen
  for(int i = 0; i < 5; i++)
  {
    u8g.drawFrame(i,i,128-2*i,64-2*i);
  }

  //sets the font for the text
  u8g.setFont(u8g_font_unifont);

  //initial screen
  if(page == 0)
  {
    u8g.drawStr(10,22,"Elektor");
    u8g.drawBitmap(70,10,2,15,c);
  }

  //visualization of the measured weight in gram and ounce
  if(page == 1 || page == 2)
  {
    u8g.drawStr(10,20,"Mass:");

    u8g.setPrintPos(10,33);
    u8g.print(gram,1);
    u8g.print(" g");
  
    u8g.setPrintPos(10,45);
    u8g.print(ounce);
    u8g.print(" oz");

    //bar graph of the weight in gram
    u8g.drawStr(101,17,"0");
    u8g.drawStr(83,56,"5kg");//for use with a different max load, change the string to the new max and change the first number(-8 for each added character)
    u8g.drawFrame(110,10,7,44);

    //makes sure the bar does not surpass the graph and makes sure the height of the bar isn't less then 1
    if(bar >= 1 && bar <= 40)
    {
      u8g.drawBox(112,12,3,bar);
    }
    else if(bar > 40)
    {
      u8g.drawBox(112,12,3,40);
      u8g.drawStr(10,57,"Too much!");
    }

    //displays wether or not the program is in hold mode
    if(page == 2)
    {
      u8g.drawStr(64,20,"Hold");
    }
  }

  //tells the user that the scale has been calibrated
  if(page >= 3 && page <= 6)
  {
    u8g.drawStr( 10, 20, "calibrating");
    if(page == 4)
    {
      u8g.drawStr( 10, 33, ".");
    }
    if(page == 5)
    {
      u8g.drawStr( 10, 33, "..");
    }
    if(page == 6)
    {
      u8g.drawStr( 10, 33, "...");
    }
  }
}

//main loop
void loop() 
{
  //measures an average ADC value of the potmeter
  for (int i = 0; i < 100; i++)
  {
    factor += analogRead(4);
  }
  //the measurement is calculated in a factor for the weight
  factor /= 100;
  factor += 15500;

  //the actual weight is calculated in gram and ounce(avoirdupoids)
  gram = (scale.get_units(10) * int(factor) / 1000);
  ounce = gram * 0.0352739619;

  //calculates the height of the bar in the graph. max height is 40 pixels and max weight is 5000g
  bar = gram*40/5000;

  //initiates the draw function for the visualisation of the weight
  u8g.firstPage();  
  do {
    draw(1);
  } while(u8g.nextPage());
  
  //resets the weight to 0
  if(button_zero) 
  {    
    button_zero = 0;
    
    scale.tare();

    for(int i = 3; i < 7; i++)
    {
      u8g.firstPage();  
      do {
        draw(i);
      } while(u8g.nextPage());
      delay(250);
    }
  }

  //holds the weight value until the button is pressed again
  if(button_hold == 1)
  {
    button_hold = 0;
    u8g.firstPage();  
    do {
      draw(2);
    } while(u8g.nextPage());
    
    delay(1000);
    while(1)
    {
      if(digitalRead(button2) == 0)
      {
        delay(20);
        break;
      }
    }
    button_hold = 0;
  }
}
